Scriptname idek:uss:USSScript extends ReferenceAlias

WorkshopParentScript Property WorkshopParent Auto Const Mandatory

; Enables the uncapping function of the respective thing
GlobalVariable Property _USSEnableFood Auto Const Mandatory
GlobalVariable Property _USSEnableWater Auto Const Mandatory
GlobalVariable Property _USSEnableScrap Auto Const Mandatory
GlobalVariable Property _USSEnableFertilizer Auto Const Mandatory
GlobalVariable Property _USSEnableCaps Auto Const Mandatory

; -1 (default) disables cap, other values set the cap on a per-settlement basis (though currently setting separate caps for each settlement is unsupported)
GlobalVariable Property _USSCapFood Auto Const Mandatory
GlobalVariable Property _USSCapWater Auto Const Mandatory
GlobalVariable Property _USSCapScrap Auto Const Mandatory

GlobalVariable Property _USSEnableAttackReducer Auto Const Mandatory
GlobalVariable Property _USSAttackReducerExponent Auto Const Mandatory
GlobalVariable Property _USSMadnessMode Auto Const Mandatory

GlobalVariable Property _USSDebugMode Auto Const Mandatory

int workshopID = -1
int dailyUpdateTimerID = 1 Const
int retryRegisterTimerID = 2 Const
int dailyPreUpdateTimerID = 3 Const
int restoreLADSTimerID = 4 Const

ObjectReference thisWorkbench
WorkshopScript thisWorkshopScript
WorkshopDataScript:WorkshopRatingKeyword[] ratings

; some constants lifted verbatim from WorkshopScript
int maxStoredFoodBase = 10 Const 					; stop producing when we reach this amount stored
int maxStoredFoodPerPopulation = 1 Const 			; increase max for each population

int maxStoredWaterBase = 5 Const 					; stop producing when we reach this amount stored
float maxStoredWaterPerPopulation = 0.25 Const 		; increase max for each population

int maxStoredScavengeBase = 100 Const 				; stop producing when we reach this amount stored
int maxStoredScavengePerPopulation = 5 Const 		; increase max for each population

int maxBrahminFertilizerProduction = 3 Const		; max fertilizer production per settlement per day

int minVendorIncomePopulation = 5 Const				; need at least this population to get any vendor income
float maxVendorIncome = 50.0 Const					; max daily vendor income from any settlement
float vendorIncomePopulationMult = 0.03 Const		; multiplier on population, added to vendor income
float vendorIncomeBaseMult = 2.0 Const				; multiplier on base vendor income

float attackChanceBase = 0.02 Const
float attackChanceResourceMult = 0.001 Const
float attackChanceSafetyMult = 0.01 Const
float attackChancePopulationMult = 0.005 Const
; end copied constants

bool firstRun = false

; used to improve the accuracy of surplus production
int lastStoredFood = 0
int lastStoredWater = 0
int lastStoredScavenge = 0

; used for attack reducer tracking, and for determining when to run 
int lastDaysSinceLastAttack = -1
int tempDaysSinceLastAttack = -1
int postPreUpdateDaysSinceLastAttack = -1
int daysLeftToDelayAttacks = -1
int currentDelayCount = 0

int maxDelayCount = 20 const
float postUpdateLADSRestoreDelay = 0.33 const

; Make some local pointers to cut down on WorkshopParent hits and allow USS daily updates to run a few frames faster
Keyword WorkshopConsumeFood
Keyword WorkshopConsumeWater
FormList WorkshopConsumeScavenge
LeveledItem WorkshopProduceWater
LeveledItem WorkshopProduceScavenge
LeveledItem WorkshopProduceVendorIncome
LeveledItem WorkshopProduceFertilizer

; Holotape Property _USSConfigHolotape Auto Const Mandatory
; Actor Property PlayerRef Auto Const Mandatory

; Fires when the quest starts up and when it resets
Event OnAliasReset()
	thisWorkbench = getReference()
		
	if (thisWorkbench != None)
		FillPointers()
		firstRun = true
		
		if (TryRegisterForUpdates()) ; Only register for updates if the alias has actually been filled (we leave some aliases open for mods and DLCs)
		
			if (_USSDebugMode.GetValue())
				Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] A quest alias reset and was filled. USS active on this workshop." )
			endif
			
		else
			StartTimerGameTime(24.0, retryRegisterTimerID)
			
			if (_USSDebugMode.GetValue())
				Debug.TraceUser("UncappedSettlementSurplus", "A quest alias reset and was filled, however a workshop ID could not be resolved. Retrying in 24 game-hours.")
			endif
		endif
	endif

EndEvent

; restore temporary changes to LADS, if necessary (will very rarely ever actually need to do anything)
Event OnAliasShutdown()
	LADSRestore()
EndEvent

bool Function TryRegisterForUpdates()
	GetWorkshopID()
	
	if (workshopID >= 0)
		RegisterForCustomEvent(WorkshopParent, "WorkshopDailyUpdate")
		return true
	endif
	
	return false
EndFunction

Event WorkshopParentScript.WorkshopDailyUpdate(WorkshopParentScript akSender, Var[] akArgs)
	if (thisWorkshopScript.OwnedByPlayer == true)
		float dui = WorkshopParent.dailyUpdateIncrement
		float waitTime = Math.Max(0.0, (dui * workshopID) - dui * 0.5)
		StartTimerGameTime(waitTime, dailyPreUpdateTimerID)
	endif
EndEvent

Event OnTimerGameTime(int aiTimerID)
	if (aiTimerID == dailyUpdateTimerID)
		if ((currentDelayCount < maxDelayCount) && (postPreUpdateDaysSinceLastAttack == thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingLastAttackDaysSince].resourceValue) as int))
			; Delay the USS update, because WorkshopScript probably hasn't run today
			
			currentDelayCount += 1
			if (_USSDebugMode.GetValue())
				Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] Tried to run on this workshop, but detected that WorkshopScript hasn't run yet - delaying (delay count: " + currentDelayCount + ")")
			endif
			
			StartTimerGameTime(utility.RandomFloat(0.4, 0.6), dailyUpdateTimerID)
		else
			currentDelayCount = 0
			DailyUpdate()
			
			; moved here, from the end of DailyUpdate(), to ensure that this runs even if DailyUpdate() errors - this was happening to some console users and I don't know why (mod conflict?)
			; if this doesn't get reset to false, the -entire- workshop daily update system will screech to a halt, and this will not ever unset itself without outside intervention
			WorkshopParent.DailyUpdateInProgress = false 
		endif
	elseif (aiTimerID == dailyPreUpdateTimerID)
		PreDailyUpdate()
	elseif (aiTimerID == restoreLADSTimerID)
		LADSRestore(1)
	elseif (aiTimerID == retryRegisterTimerID)
		FillPointers() ; pointers probably don't need refilling, but do it anyway
		if (TryRegisterForUpdates())
			if (_USSDebugMode.GetValue())
				Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] A quest alias reset and was filled, and registered for updates on the second try. USS active on this workshop.")
			endif
		else
			if (_USSDebugMode.GetValue())
				Debug.TraceUser("UncappedSettlementSurplus", "A quest alias reset and was filled, however a workshop ID could not be resolved on the second try. This is normal to happen once or twice.")
			endif
		endif
	endif
EndEvent

; Collect some data before WorkshopScript runs
Function PreDailyUpdate()
	; we can accurately determine if WorkshopScript has finished running by whether LADS has changed or not
	; however, if this workshop has attacks disabled (this is normally only true for The Castle), this never changes, so special handle (read: wait longer and hope for the best)
	lastDaysSinceLastAttack = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingLastAttackDaysSince].resourceValue) as int
	int ARSetting = _USSEnableAttackReducer.GetValue() as int
	bool debugMode = _USSDebugMode.GetValue() as bool
	
	if (debugMode)
		Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] Beginning pre-daily update at this workshop...")
	endif
	
	; also, set LADS modifications here, so WorkshopScript.DailyUpdate() reads our values instead of the real LADS value,
	; and by doing so we can manipulate WorkshopScript.DailyUpdate() - these mods get restored at the end of DailyUpdateAttackLimiter()
	if (ARSetting)
		if (ARSetting == 2 || ARSetting == 4)
			if (ARSetting == 2)
				LADSSet(5, true)
			else
				LADSSet(4, false)
			endif
			
			if (debugMode)
				Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] AR: LastAttackDaysSince rating temporarily set to 5, because attacks are disabled in the config.")
			endif
		elseif (daysLeftToDelayAttacks > 0)
			if (ARSetting == 1)
				LADSSet(5, true)
			else
				LADSSet(5, false)
			endif
			
			daysLeftToDelayAttacks -= 1
			
			if (debugMode)
				Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] AR: LastAttackDaysSince rating temporarily set to 5, because attack suppressor is active for this settlement - " + daysLeftToDelayAttacks + " attack-free days left.")
			endif
		elseif (_USSMadnessMode.GetValue())
			LADSSet(7, true)
			
			if (debugMode)
				Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + " AR: LastAttackDaysSince rating temporarily set to 7, because madness mode is enabled in the config.")
			endif
		endif
		
		postPreUpdateDaysSinceLastAttack = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingLastAttackDaysSince].resourceValue) as int
	else
		; don't bother reading the actor value again because that's slow, and we didn't change it anyway
		postPreUpdateDaysSinceLastAttack = lastDaysSinceLastAttack
	endif
	
	lastStoredFood = thisWorkbench.GetItemCount(WorkshopParent.WorkshopConsumeFood)
	lastStoredWater = thisWorkbench.GetItemCount(WorkshopParent.WorkshopConsumeWater)
	lastStoredScavenge = thisWorkbench.GetItemCount(WorkshopParent.WorkshopConsumeScavenge)
	
	StartTimerGameTime(WorkshopParent.dailyUpdateIncrement, dailyUpdateTimerID)
	
	if (debugMode)
		Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] Pre-daily update complete at this workshop. Counted...	Food: " + lastStoredFood + "		Water: " + lastStoredWater + "		Scrap: " + lastStoredScavenge + "		LADS: " + lastDaysSinceLastAttack)
	endif
EndFunction

; So basically we have to simulate most of the calculations WorkshopScript does until DailyUpdateSurplusResources(), then work some hacky magic with the numbers that come out later
Function DailyUpdate()
	bool debugMode = _USSDebugMode.GetValue() as bool
	if (debugMode)
		Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] Beginning daily update at this workshop...")
	endif
	
	GetWorkshopID()
	
	; wait for update lock to be released
	while (WorkshopParent.DailyUpdateInProgress)
		utility.wait(utility.randomFloat(0.4,0.6)) 
	endwhile
		
	; tried this without the global update lock and found it would trigger the thread sync issue as with UFO4P #20295, and others
	WorkshopParent.DailyUpdateInProgress = true

	; create local pointer to WorkshopRatings array to speed things up
	WorkshopScript:DailyUpdateData updateData = new WorkshopScript:DailyUpdateData

	updateData.totalPopulation = thisWorkbench.GetBaseValue(ratings[WorkshopParent.WorkshopRatingPopulation].resourceValue) as int
	updateData.robotPopulation = thisWorkbench.GetBaseValue(ratings[WorkshopParent.WorkshopRatingPopulationRobots].resourceValue) as int
	updateData.brahminPopulation = thisWorkbench.GetBaseValue(ratings[WorkshopParent.WorkshopRatingBrahmin].resourceValue) as int
	updateData.unassignedPopulation = thisWorkbench.GetBaseValue(ratings[WorkshopParent.WorkshopRatingPopulationUnassigned].resourceValue) as int

	updateData.vendorIncome = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingVendorIncome].resourceValue) * vendorIncomeBaseMult
	updateData.currentHappiness = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingHappiness].resourceValue)

	updateData.damageMult = 1 - thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingDamageCurrent].resourceValue)/100.0
	updateData.productivity = WorkshopParent.Workshops[workshopID].GetProductivityMultiplier(ratings)
	updateData.availableBeds = thisWorkbench.GetBaseValue(ratings[WorkshopParent.WorkshopRatingBeds].resourceValue) as int
	updateData.shelteredBeds = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingBeds].resourceValue) as int
	updateData.bonusHappiness = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingBonusHappiness].resourceValue) as int
	updateData.happinessModifier = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingHappinessModifier].resourceValue) as int
	updateData.safety = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingSafety].resourceValue) as int
	updateData.safetyDamage = thisWorkbench.GetValue(WorkshopParent.GetDamageRatingValue(ratings[WorkshopParent.WorkshopRatingSafety].resourceValue)) as int
	updateData.totalHappiness = 0.0

	ObjectReference containerRef = thisWorkbench.GetContainer()
	if (!containerRef)
		if (debugMode)
			Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] This workshop did not have a container, skipping daily update.")
		endif
	
		WorkshopParent.DailyUpdateInProgress = false
		return
	endif
	
	; bit of a hack but saves several hundred lines of code
	thisWorkshopScript.DailyUpdateProduceResources(ratings, updateData, containerRef, false) 
	thisWorkshopScript.DailyUpdateConsumeResources(ratings, updateData, containerRef, false)

	DailyUpdateSurplusResources(updateData, containerRef)
	
	if (_USSEnableAttackReducer.GetValue())
		DailyUpdateAttackLimiter(updateData)
	else
		LADSRestore(1)	; attack reducer has been disabled, so restore LADS if necessary
	endif
	
	; clear update lock
	; WorkshopParent.DailyUpdateInProgress = false ; moved until just after the DailyUpdate() call to prevent everything from breaking in case DailyUpdate() decides to error and return early
	firstRun = false
	
	if (debugMode)
		Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] Daily update complete at this workshop.")
	endif
EndFunction

Function DailyUpdateSurplusResources(WorkshopScript:DailyUpdateData updateData, ObjectReference containerRef)
	int currentStoredFood = containerRef.GetItemCount(WorkshopConsumeFood)
	int currentStoredWater = containerRef.GetItemCount(WorkshopConsumeWater)
	int currentStoredScavenge = containerRef.GetItemCount(WorkshopConsumeScavenge)
	
	int addedFood = 0
	int addedWater = 0
	int addedScavenge = 0
	int addedFertilizer = 0
	int addedCaps = 0

	; Note: The following line comes from the base script: Bethesda is trying to run GetItemCount on a leveled list, which does not work and returns 0
	; As a result, currentStoredFertilizer is always 0, so the vanilla script does not cap it like intended and I don't have to try uncapping it
	;int currentStoredFertilizer = containerRef.GetItemCount(WorkshopParent.WorkshopProduceFertilizer)

	; In this section I do the opposite of what the vanilla script does: if I calculate that production likely ran last cycle 
	; in the vanilla script then do nothing,  and if it didn't, make up for it by producing what it would have had it ran
	
	; Approximately add the food the vanilla script didn't add
	if ((updateData.foodProduction > 0) && (_USSEnableFood.GetValue()))
		updateData.foodProduction = math.Floor(updateData.foodProduction * updateData.productivity)
		
		; Is food production still non-zero? -- is there more water than the pre-update? if so, don't produce, because WorkshopScript probably already did
		if ((updateData.foodProduction > 0) && (currentStoredFood <= lastStoredFood))
			int maxFood = maxStoredFoodBase + maxStoredFoodPerPopulation * updateData.totalPopulation
			
			; Is there enough food that the vanilla script has stopped producing?
			if (currentStoredFood > maxFood)
				; if currentStoredFood - updateData.foodProduction <= maxFood && currentStoredFood >= updateData.foodProduction * multiAddFood * 0.9

				; > _USSCapFood < 0 = unlimited
				if (!(_USSCapFood.GetValue() < 0) && (updateData.foodProduction + currentStoredFood > _USSCapFood.GetValue()))
					; The user set a food cap, so modify production as needed now
					updateData.foodProduction =  Math.floor(_USSCapFood.GetValue()) - currentStoredFood
				endif

				if (updateData.foodProduction > 0)
					WorkshopParent.ProduceFood(thisWorkshopScript, updateData.foodProduction)
					addedFood = updateData.foodProduction
				endif
				
			endif
		endif
	endif	
	
	; Approximately add the water the vanilla script didn't add
	if ((updateData.waterProduction > 0) && (_USSEnableWater.GetValue()))
		
		; is there more water than the pre-update? if so, don't produce, because WorkshopScript probably already did
		if (currentStoredWater <= lastStoredWater)
			int maxWater =  maxStoredWaterBase + math.floor(maxStoredWaterPerPopulation * updateData.totalPopulation)
			
			; Is there enough water that the vanilla script has stopped producing?
			if (currentStoredWater > maxWater)
				; if currentStoredWater - updateData.waterProduction <= maxWater && currentStoredWater >= updateData.waterProduction * multiAddWater * 0.9

				if (!(_USSCapWater.GetValue() < 0) && (updateData.waterProduction + currentStoredWater > _USSCapWater.GetValue()))
					; The user set a water cap, so modify production as needed now
					updateData.waterProduction =  Math.floor(_USSCapWater.GetValue()) - currentStoredWater
				endif

				if (updateData.waterProduction > 0)
					containerRef.AddItem(WorkshopProduceWater, updateData.waterProduction)
					addedWater = updateData.waterProduction
				endif
				
			endif
		endif
	endif
	
	if (_USSEnableScrap.GetValue())
	
		int scavengePopulation = (updateData.unassignedPopulation - thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingDamagePopulation].resourceValue)) as int
		int scavengeProductionGeneral = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingScavengeGeneral].resourceValue) as int
		int scavengeAmount = math.Ceiling(scavengePopulation * updateData.productivity * updateData.damageMult + scavengeProductionGeneral*updateData.productivity)
		int maxScavenge = maxStoredScavengeBase + maxStoredScavengePerPopulation * updateData.totalPopulation
		
		if ((scavengeAmount > 0) && (currentStoredScavenge <= lastStoredScavenge) && (currentStoredScavenge > maxScavenge))
			; if (currentStoredScavenge - scavengeAmount <= maxScavenge) && (currentStoredScavenge >= scavengeAmount * multiAddScrap * 0.9)
			
			if (!(_USSCapScrap.GetValue() < 0) && (scavengeAmount + currentStoredScavenge > _USSCapScrap.GetValue()))
				; The user set a scavenge cap, so modify production as needed now
				scavengeAmount = Math.floor(_USSCapScrap.GetValue()) - currentStoredScavenge
			endif

			if (scavengeAmount > 0)
				containerRef.AddItem(WorkshopProduceScavenge, scavengeAmount)
				addedScavenge = scavengeAmount
			endif

		endif
	endif

	; Add fertilizer past the brahmin cap (the vanilla script is uncapped anyway because of a bug)
	if (_USSEnableFertilizer.GetValue() && (updateData.brahminPopulation > maxBrahminFertilizerProduction))
		int fertilizerProduction = updateData.brahminPopulation - maxBrahminFertilizerProduction

		containerRef.AddItem(WorkshopProduceFertilizer, fertilizerProduction)
		addedFertilizer = fertilizerProduction
	endif

	; Add caps past cap... cap
	if (updateData.vendorIncome > 0 && _USSEnableCaps.GetValue())
		int vendorIncomeFinal = 0

		; get linked population with productivity excluded
		float linkedPopulation = WorkshopParent.GetLinkedPopulation(thisWorkshopScript, false)
		float vendorPopulation = linkedPopulation + updateData.totalPopulation

		; only get income if population >= minimum
		if (vendorPopulation >= minVendorIncomePopulation)
			linkedPopulation = WorkshopParent.GetLinkedPopulation(thisWorkshopScript, true)

			vendorPopulation = updateData.totalPopulation * updateData.productivity + linkedPopulation

			float incomeBonus = updateData.vendorIncome * vendorIncomePopulationMult * vendorPopulation

			updateData.vendorIncome = updateData.vendorIncome + incomeBonus

			vendorIncomeFinal = math.Ceiling(updateData.vendorIncome)

			; USS: add caps past the cap
			if (vendorIncomeFinal > maxVendorIncome)
				vendorIncomeFinal = Math.floor(vendorIncomeFinal - maxVendorIncome)
			else
				vendorIncomeFinal = 0
			endif

			if (vendorIncomeFinal > 0)
				containerRef.AddItem(WorkshopProduceVendorIncome, vendorIncomeFinal)
				addedCaps = vendorIncomeFinal
			endif

		endif
	endif
	
	if (_USSDebugMode.GetValue())
		Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] Surplus calculation complete at this workshop. Added...	Food: " + addedFood + "		Water: " + addedWater + "		Scrap: " + addedScavenge + "		Fertilizer: " + addedFertilizer + "		Caps: " + addedCaps)
	endif
EndFunction

Function DailyUpdateAttackLimiter(WorkshopScript:DailyUpdateData updateData)	
	int daysSinceLastAttack = thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingLastAttackDaysSince].resourceValue) as int
	
	if (_USSMadnessMode.GetValue() >= 1.9)
		int foodRating = WorkshopParent.Workshops[workshopID].GetTotalFoodRating(ratings)
		int waterRating = WorkshopParent.Workshops[workshopID].GetTotalWaterRating(ratings)
	
		WorkshopParent.TriggerAttack(WorkshopParent.Workshops[workshopID], WorkshopParent.CalculateAttackStrength(foodRating, waterRating))
		
		if (_USSDebugMode.GetValue())
			Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID +"] Triggered an attack at this workshop, because extra madness mode is enabled.")
		endif
	endif
	
	if ((daysSinceLastAttack < postPreUpdateDaysSinceLastAttack && tempDaysSinceLastAttack == -1) || firstRun)
		; current LADS less than last LADS, and there's no temporary change to LADS at this settlement, which means an attack must have happened today
		; alternatively this is the first run, so run this block even though there's no attack
	
		int foodRating = WorkshopParent.Workshops[workshopID].GetTotalFoodRating(ratings)
		int waterRating = WorkshopParent.Workshops[workshopID].GetTotalWaterRating(ratings)
		
		; calculate attack chance used in WorkshopScript
		float attackChance = attackChanceBase + attackChanceResourceMult * (foodRating + waterRating) - attackChanceSafetyMult * updateData.safety - attackChancePopulationMult * updateData.totalPopulation
		
		if (attackChance < attackChanceBase)
			attackChance = attackChanceBase
		endif
		
		;/ ---- 
		; Our formula here is floor(max(0,(x^0.66 + sqrt(x)) * min(1, b^0.2) * random(0.66, 1.33) - a)) ---- ( <= 1.3.1 formula also subtracted 7 from a)
		; ...where 'a' is days since last attack, x is defense and b is the daily attack chance at the moment of calculation.
		; The first part gives us a curve for defense to attack delay where doubling defense increases time between raids by about 1.5x.
		; The second part gives us a pseudo-logarithmic scale based on attack chance to mutiply the effects of the first part against, so as not to excessively reduce attacks to settlements
		; that already have a low attack chance. This kicks in like 80% when attack chance is above roughly once per week, and rapidly drops off when it's less than that. It's not perfect,
		; but it should keep attacks from being excessively reduced when the low attack chance would make them rare anyway.
		; The third part is a little random variation so that attacks won't happen on an exact schedule.
		; Last, we adjust for days passed since the last attack happened in case this runs a day (or two+) late, make sure our value isn't negative, and round it down into an int
		/;
		daysLeftToDelayAttacks = Math.floor(Math.max(0, Math.pow(updateData.safety, _USSAttackReducerExponent.GetValue()) + Math.sqrt(updateData.safety) * Math.min(1, Math.pow(attackChance, 0.2)) * Utility.RandomFloat(0.66, 1.33) - daysSinceLastAttack))
		
		if (firstRun)
			; this is the first run, so multiply daysLeftToDelayAttacks by a random amount
			; this won't really do anything on new games, but will affect existing games, and in particular those where USS is just being reset
			daysLeftToDelayAttacks = Math.floor(daysLeftToDelayAttacks * Utility.RandomFloat(0, 1))
			
			if (_USSDebugMode.GetValue())
				Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] AR: First run on this workshop. Setting 'suppression days' to " + daysLeftToDelayAttacks + " because defense is " + updateData.safety + " and attackChance is " + attackChance + ".")
			endif
		else
			if (_USSDebugMode.GetValue())
				Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] AR: Attack detected at this workshop. Setting 'suppression days' to " + daysLeftToDelayAttacks + " because defense is " + updateData.safety + " and attackChance is " + attackChance + ".")
			endif
		endif
	else
		; reset LADS on workbench to value from before PreDailyUpdate(), and add a day
		; only do this if there was no attack today
		; LADSRestore(daysSinceLastAttack - tempDaysSinceLastAttack)
		
		; need to delay this slightly, so do it after another timer
		StartTimerGameTime(postUpdateLADSRestoreDelay, restoreLADSTimerID)
	endif
EndFunction 

;/ Why do I do it this way, by hacking DaysSinceLastAttack instead of just disabling attacks using the AllowAttacks script property?
; This way wears off over time. If USS gets uninstalled, or reset, or whatever, I -really- do not want to permanently break attacks on
; all affected settlements, and it's likely that vanilla scripts fiddle with the AllowAttacks property sometimes, which could also break things.
; Doing it this way, if USS gets uninstalled, or reset, or stops working (etc), worst-case-scenario is that the save goes back to normal
; after one game-week.
; 
; I know it's hacky, and might mildly conflict with e.g. Settlement Management Software, but it really is preferable to alternatives
/;

; set LastAttackDaysSince, with or without provisions for setting it back in DailyUpdate()
Function LADSSet(int i, bool restoreLater = false)
	; WorkshopParent.SetResourceData(ratings[WorkshopParent.WorkshopRatingLastAttackDaysSince].resourceValue, thisWorkshopScript, i as float)
	; The vanilla script function here doesn't work on all games for some stupid reason, so use the native function instead
	thisWorkbench.SetValue(ratings[WorkshopParent.WorkshopRatingLastAttackDaysSince].resourceValue, i as float)
	
	if (restoreLater)
		tempDaysSinceLastAttack = i
	else
		tempDaysSinceLastAttack = -1
	endif
	
	;if (_USSDebugMode.GetValue())
	;	Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] LADS set to a value of " + thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingLastAttackDaysSince].resourceValue) as int + ", restoreLater = " + restoreLater)
	;endif
EndFunction

; reset LastAttackDaysSince to the last value set by LADSSet(), plus whatever value it gets passed
Function LADSRestore(int i = 0)
	if (tempDaysSinceLastAttack != -1)
		LADSSet(lastDaysSinceLastAttack + i, false)
	endif
	
	if (_USSDebugMode.GetValue())
		Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] LADS restored to a value of " + (thisWorkbench.GetValue(ratings[WorkshopParent.WorkshopRatingLastAttackDaysSince].resourceValue) as int + i) + " at this workshop.")
	endif
EndFunction

; A bunch of pointers, mostly to WorkshopParent, to speed up execution slightly
; These should never change for the lifetime of this alias, so setting them once is fine
; This function may safely be called from other scripts if e.g. we added new pointers and need to fill them, but don't want to do a full mod reset
Function FillPointers()
	thisWorkbench = getReference()
	thisWorkshopScript = thisWorkbench as WorkshopScript
	GetWorkshopID()
	
	ratings = WorkshopParent.WorkshopRatings
	
	WorkshopConsumeFood = WorkshopParent.WorkshopConsumeFood
	WorkshopConsumeWater = WorkshopParent.WorkshopConsumeWater
	WorkshopConsumeScavenge = WorkshopParent.WorkshopConsumeScavenge
	
	WorkshopProduceWater = WorkshopParent.WorkshopProduceWater
	WorkshopProduceScavenge = WorkshopParent.WorkshopProduceScavenge
	WorkshopProduceVendorIncome = WorkshopParent.WorkshopProduceVendorIncome
	WorkshopProduceFertilizer = WorkshopParent.WorkshopProduceFertilizer
	
	if (_USSDebugMode.GetValue())
		Debug.TraceUser("UncappedSettlementSurplus", "[ID: " + workshopID + "] Finished filling pointers for USSScript at this workshop.")
	endif
EndFunction

int Function GetWorkshopID()
	if (workshopID < 0)
		workshopID = WorkshopParent.GetWorkshopID(thisWorkshopScript)
		
		;if (workshopID < 0); couldn't get workshop ID
		;	DebugMsgBox("USS Debug: A workshop was unable to get an ID from WorkshopParent - object is " + thisWorkbench, 1.0)
		;endif
	endif
	
	return workshopID
EndFunction
