Scriptname idek:uss:USSTempAttackDisableScript extends Quest

GlobalVariable Property _USSEnableAttackReducer	Auto Const Mandatory

WorkshopParentScript Property WorkshopParent	Auto Const Mandatory
GlobalVariable Property _USSDebugMode			Auto Const Mandatory

float[] LADSArr = None
int LADSAV = 0
WorkshopDataScript:WorkshopRatingKeyword[] WorkshopRatings = None

Event OnQuestInit()
	DisableAttacks()
EndEvent

Event OnQuestShutdown()
	RestoreAttacks()
EndEvent

Event WorkshopParentScript.WorkshopDailyUpdate(WorkshopParentScript akSender, Var[] akArgs)
	if (_USSDebugMode.GetValue())
		Debug.TraceUser("UncappedSettlementSurplus", "Temp Attack Reducer: WorkshopDailyUpdate event received, re-enabling attacks...")
	endif
	Stop()
EndEvent

Function DisableAttacks()
	bool debugMode = _USSDebugMode.GetValue() as bool
	if (_USSEnableAttackReducer.GetValue() > 0)
		if (debugMode)
			Debug.TraceUser("UncappedSettlementSurplus", "Temp Attack Reducer: Quest started. Halting attacks until the start of the next daily update cycle...")
		endif
		
		WorkshopRatings = WorkshopParent.WorkshopRatings
		LADSAV = WorkshopParent.WorkshopRatingLastAttackDaysSince
		
		int workshopLen = WorkshopParent.Workshops.Length
		LADSArr = new float[workshopLen]
		for int i = 0 to workshopLen - 1
			WorkshopScript thisWorkshopScript = WorkshopParent.Workshops[i]
			float thisLADS = thisWorkshopScript.GetValue(WorkshopRatings[LADSAV].resourceValue)
			
			if (debugMode)
				Debug.TraceUser("UncappedSettlementSurplus", "Temp Attack Reducer: Workshop ID " + i + " has LADS value of " + thisLADS)
			endif
			
			if (thisLADS < 6 || thisWorkshopScript.OwnedByPlayer == false)
				; this settlement already has at least one attack-free day coming up, so there's no need to explicitly delay attacks
				; alternatively, the player doesn't own it, so skip it
				LADSArr[i] = -1
				
				if (debugMode)
					Debug.TraceUser("UncappedSettlementSurplus","Temp Attack Reducer: Workshop ID " + i + " skipped. Owned? " + thisWorkshopScript.OwnedByPlayer + "; LADS: " + thisLADS)
				endif
			else
				LADSArr[i] = thisLADS
				
				if (debugMode)
					Debug.TraceUser("UncappedSettlementSurplus", "Temp Attack Reducer: Setting Workshop ID " + i + " LADS to 1")
				endif
				
				thisWorkshopScript.SetValue(WorkshopRatings[LADSAV].resourceValue, 1)
			endif
		endfor
		
		RegisterForCustomEvent(WorkshopParent, "WorkshopDailyUpdate")
		
		if (debugMode)
			Debug.TraceUser("UncappedSettlementSurplus","Temp Attack Reducer: Attacks successfully halted.")
		endif
	else
		; don't bother doing anything if USS isn't configured to suppress attacks anyway
		
		if (debugMode)
			Debug.TraceUser("UncappedSettlementSurplus", "Temp Attack Reducer: Quest started, but the attack reducer is disabled via config.")
		endif
		
		Stop()
	endif
EndFunction

Function RestoreAttacks()
	bool debugMode = _USSDebugMode.GetValue() as bool
	if (LADSArr != None)
		if (debugMode)
			Debug.TraceUser("UncappedSettlementSurplus", "Temp Attack Reducer: Restoring cached LADS values...")
		endif
		
		for int i = 0 to LADSArr.Length - 1
			if (LADSArr[i] != -1)
				LADSArr[i] = LADSArr[i] + WorkshopParent.Workshops[i].GetValue(WorkshopRatings[LADSAV].resourceValue) - 1
				
				if (debugMode)
					Debug.TraceUser("UncappedSettlementSurplus", "Temp Attack Reducer: Restoring Workshop ID " + i + " LADS to " + LADSArr[i])
				endif
				
				WorkshopParent.Workshops[i].SetValue(WorkshopRatings[LADSAV].resourceValue, LADSArr[i])
			endif
		endfor
		
		if (debugMode)
			Debug.TraceUser("UncappedSettlementSurplus", "Temp Attack Reducer: Cached LADS values restored. Attacks are active again (unless disabled in the USS config).")
		endif
		
		UnregisterForCustomEvent(WorkshopParent, "WorkshopDailyUpdate")
		LADSArr = None
	endif
	
	if (debugMode)
		Debug.TraceUser("UncappedSettlementSurplus", "Temp Attack Reducer: Quest has shut down.")
	endif
EndFunction
