Scriptname idek:uss:USSUpdateScript extends ReferenceAlias

GlobalVariable Property _USSVersion 				Auto Const Mandatory
GlobalVariable Property _USSVersionCurrent			Auto Const Mandatory
GlobalVariable Property _USSDebugMode 				Auto Const Mandatory
GlobalVariable Property _USSLastWorkshopCount		Auto Const Mandatory

Quest Property _USSReset							Auto Const Mandatory
Quest Property _USSQuest							Auto Const Mandatory

WorkshopParentScript Property WorkshopParent		Auto Const Mandatory

Message Property _USSMessageUpdate127				Auto Const Mandatory
Message Property _USSMessageUpdateGeneral			Auto Const Mandatory
Message Property _USSMessageUpdateUnknown			Auto Const Mandatory
Message Property _USSMessageAutoResetNew			Auto Const Mandatory
Message Property _USSMessageAutoResetRemoved		Auto Const Mandatory

Holotape Property _USSConfigHolotape				Auto Const Mandatory

Event OnAliasReset()
	DoUpdate()
EndEvent

Event OnPlayerLoadGame()
	DoUpdate()
EndEvent

Function DoUpdate()
	Debug.OpenUserLog("UncappedSettlementSurplus")
	Debug.TraceUser("UncappedSettlementSurplus", "Log opened. Note that not much will print out here unless USS's debug mode is enabled (either via holotape or typing \"set _USSDebugMode to 1\" in the console.")
	
	float versionLast = _USSVersion.GetValue()
	float versionCurrent = _USSVersionCurrent.GetValue()
	bool hasAlreadyReset = false
	
	if (versionCurrent != versionLast)
		Debug.TraceUser("UncappedSettlementSurplus", "Update detected! Running update script... Last version = " + versionLast + "; Current version = " + versionCurrent + ".")
		
		Utility.Wait(3) ; delay slightly so notifications don't display while the screen is still fading in
		if (versionLast == 0)
			; queue a reset if necessary (this also runs on new games)
			hasAlreadyReset = Update140()
			RemoveOldHolotapes()
			_USSMessageUpdateGeneral.Show()
		elseif (versionLast <= 1.41)
			; no need to do this for updates from 1.3.x or below, because a full reset triggers in that case anyway
			FillPointers()
			RemoveOldHolotapes()
			_USSMessageUpdateGeneral.Show()
		elseif (versionLast <= 1.45)
			RemoveOldHolotapes()
			_USSMessageUpdateGeneral.Show()
		elseif (versionLast < 1.5)
			; still 1.4.x, probably a minor update, so don't really do anything unless necessary
			_USSMessageUpdateGeneral.Show()
		else
			; version unrecognized - maybe downgrading?
			_USSMessageUpdateUnknown.Show()
		endif
		
		_USSVersion.SetValue(versionCurrent)
	else
		Debug.TraceUser("UncappedSettlementSurplus", "No updates this load.")
	endif
	
	int workshopCount = WorkshopParent.Workshops.Length
	int workshopCountLast = _USSLastWorkshopCount.GetValue() as int
	if (workshopCount != workshopCountLast)
		Debug.TraceUser("UncappedSettlementSurplus", "Added/removed workshop detected. Old workshop count: " + workshopCountLast + "; New workshop count: " + workshopCount + ".")
		if (workshopCountLast != 0 && hasAlreadyReset == False)
			Utility.Wait(5)
			
			if (workshopCount > workshopCountLast)
				_USSMessageAutoResetNew.Show()
			else
				_USSMessageAutoResetRemoved.Show()
			endif
			
			Debug.TraceUser("UncappedSettlementSurplus", "Running reset script...")
			_USSReset.Start()
		else
			Debug.TraceUser("UncappedSettlementSurplus", "Reset skipped. Already reset this run? " + hasAlreadyReset + ". Last workshop count 0? " + workshopCountLast)
		endif
		
		_USSLastWorkshopCount.SetValue(workshopCount as Float)
	else
		Debug.TraceUser("UncappedSettlementSurplus", "No new or removed workshops this load.")
	endif
EndFunction

bool Function Update140()
	; last version installed = 1.3.1 or below, OR new install
	; detect if aliases are correctly filled, if not do a reset
	bool debugMode = _USSDebugMode.GetValue() as bool
	if (debugMode)
		Debug.TraceUser("UncappedSettlementSurplus", "Checking if aliases are properly filled...")
	endif
	
	bool doReset = true
	ReferenceAlias workshopAlias
	int i = 1
	int filledCount = 0
	
	while (i <= 100 && doReset)
		workshopAlias = _USSQuest.GetAlias(i) as ReferenceAlias
		if (workshopAlias.GetReference() != None)
			filledCount += 1
		endif
		
		if filledCount > 15
			doReset = false
		endif

		i += 1
	endwhile

	if (doReset)
		if (debugMode)
			Debug.TraceUser("UncappedSettlementSurplus", "Aliases are not properly filled (fill count: " + i + "). Running reset script...")
		endif
		
		_USSMessageUpdate127.Show()
		_USSReset.Start()
		return true
	else
		if (debugMode)
			Debug.TraceUser("UncappedSettlementSurplus", "Aliases are fine (fill count: " + i + ", no reset triggered.")
		endif
		
		return false
	endif
EndFunction

Function FillPointers()
	; added some extra pointers for speed in 1.4.2, so make sure those get filled or everything will be borked
	bool debugMode = _USSDebugMode.GetValue() as bool
	if (debugMode)
		Debug.TraceUser("UncappedSettlementSurplus", "Filling local USSScript pointers...")
	endif
	
	int i = 1
	idek:uss:USSScript USSScript
	ObjectReference workbench
	
	while (i <= 100)
		USSScript = _USSQuest.GetAlias(i) as idek:uss:USSScript
		workbench = USSScript.GetReference()
		if (workbench != None)
			USSScript.FillPointers()
		endif
		
		i += 1
	endwhile
	
	if (debugMode)
		Debug.TraceUser("UncappedSettlementSurplus", "Pointers have been filled.")
	endif
EndFunction

Function RemoveOldHolotapes()
	; changed how holotape recovery works, so remove old ones from workbenches in case USS placed them everywhere
	bool debugMode = _USSDebugMode.GetValue() as bool
	if (debugMode)
		Debug.TraceUser("UncappedSettlementSurplus", "Cleaning up old holotapes...")
	endif
	
	int i = 1
	idek:uss:USSScript USSScript
	WorkshopScript wsScript
	ObjectReference workbench
	
	while (i <= 100)
		USSScript = _USSQuest.GetAlias(i) as idek:uss:USSScript
		wsScript = USSScript.GetReference() as WorkshopScript
		if (wsScript != None)
			workbench = wsScript.GetContainer()
			if (workbench != None)
				workbench.RemoveItem(_USSConfigHolotape, -1)
			endif
		endif
		
		i += 1
	endwhile
	
	if (debugMode)
		Debug.TraceUser("UncappedSettlementSurplus", "Holotape cleaning complete.")
	endif
EndFunction
